
local PoleFlag = ...

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	PoleFlag:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		Mover = { DataType = "BUILDING_PART_MOVER" },
		ConstructorData = {
                     DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                     CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                }
	})
end

function registerColor(_colorFbxName, _colorAssetId)

    PoleFlag:registerAssetProcessor("models/flags/poleFlag" .. _colorFbxName .. ".fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
    })

-- Register common prefab nodes
    PoleFlag:registerAssetId("models/flags/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_0_RootPart", "STONE_GATE_V5_POLE_FLAG" .. _colorAssetId .. "_ROOT_PREFAB")
    PoleFlag:registerAssetId("models/flags/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_3_TopPart", "STONE_GATE_V5_POLE_FLAG" .. _colorAssetId .. "_TOP_PREFAB")
    PoleFlag:registerAssetId("models/flags/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_2_Tiling2Part", "STONE_GATE_V5_POLE_FLAG" .. _colorAssetId .. "_TILING2_PREFAB")
    PoleFlag:registerAssetId("models/flags/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_1_Tiling3Part", "STONE_GATE_V5_POLE_FLAG" .. _colorAssetId .. "_TILING3_PREFAB")

    local defaultNodePrefixList = {
	"STONE_GATE_V5_POLE_FLAG" .. _colorAssetId .. "_TOP",
	"STONE_GATE_V5_POLE_FLAG" .. _colorAssetId .. "_TILING2",
	"STONE_GATE_V5_POLE_FLAG" .. _colorAssetId .. "_TILING3"
    }

-- Register simple building part assets
    for i, nodePrefix in ipairs(defaultNodePrefixList) do
	registerDefaultBuildingPart(nodePrefix)
    end

-- Register the pole flag part
    PoleFlag:register({
	DataType = "BUILDING_PART",
	Id = "STONE_GATE_V5_POLE_FLAG" .. _colorAssetId .. "_ROOT_PART",
	Name = "STONE_GATE_V5_POLE_FLAG" .. _colorAssetId .. "_ROOT_NAME",
	Description = "STONE_GATE_V5_POLE_FLAG" .. _colorAssetId .. "_ROOT_DESC",
	Category = "DECORATION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = "STONE_GATE_V5_POLE_FLAG" .. _colorAssetId .. "_ROOT_PREFAB",
		EndPart = "STONE_GATE_V5_POLE_FLAG" .. _colorAssetId .. "_TOP_PART",
		FillerList = {
			"STONE_GATE_V5_POLE_FLAG" .. _colorAssetId .. "_TILING2_PART"
                },
		BasementFillerList = {
			"STONE_GATE_V5_POLE_FLAG" .. _colorAssetId .. "_TILING3_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = true
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
                        { Resource = "WOOD", Quantity = 2 },
			{ Resource = "WOOLEN_CLOTH", Quantity = 2 }
		}
	}
    })

-- Override building part components properties (generated by the building asset processor) to add building zones

    PoleFlag:registerPrefabComponent("models/flags/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_0_RootPart", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes (here only on the roofs)
	BuildingZone = { 0, 0 }
    })

-- Particle system
    PoleFlag:registerAssetId("models/flags/Flag" .. _colorFbxName .. ".png", "STONE_GATE_V5_FLAG" .. _colorAssetId .. "_TEXTURE")

    PoleFlag:register({
	DataType = "MATERIAL",
	Id = "STONE_GATE_V5_FLAG" .. _colorAssetId .. "_MATERIAL",
        AlbedoColor = COL_WHITE,
	AlbedoTexture = "STONE_GATE_V5_FLAG" .. _colorAssetId .. "_TEXTURE"
    })

    PoleFlag:register({
	DataType = "PARTICLE_SYSTEM",
	Id = "STONE_GATE_V5_FLAG" .. _colorAssetId .. "_PARTICLE_SYSTEM",
	Material = "STONE_GATE_V5_FLAG" .. _colorAssetId .. "_MATERIAL",
	BillboardBehavior = "FACE_CAMERA_Y_ALIGN",
	Duration = 1,
	Looping = true,
	Delay = { 0, 0 },
	LifeTime = 1,
	StartSpeed = { 0, 0 },
	StartSize = { 2.6, 2.6 },
	StartRotation = { 0, 0 },
	StartColor = { 1, 1, 1, 1 },
	GravityModifier = 0,
	RateOverTime = 2,
	BurstList = {},
	Shape = {
		DataType = "PARTICLE_EMITTER_SHAPE_BOX",
		Size = { 0, 0.001, 0 },
                Rotation = { 0, 0, 0 },
                Position = { 0, 0, 0 }
	},
	LinearVelocity = {
		DataType = "PARTICLE_FLOAT3_VALUE_CONSTANT_RANDOM",
		ValueA = { 0, 0, 0 },
		ValueB = { 0, 0, 0 },
	},
	RotationOverLifetime = { 0, 0 },
    })

-- Register air vent particle system (flags)
    PoleFlag:registerAssetId("models/flags/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_3_TopPart/Flag" .. _colorFbxName .. "Emitter", "STONE_GATE_V5_FLAG" .. _colorAssetId .. "_EMITTER_PREFAB")

    PoleFlag:registerPrefabComponent("models/flags/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_3_TopPart", {
	DataType = "COMP_BUILDING_PART",
	FeedbackComponentListToActivate = {
                 { "STONE_GATE_V5_FLAG" .. _colorAssetId .. "_EMITTER_PREFAB", "COMP_PARTICLE_EMITTER_TOGGLE" }
	}
    })

    PoleFlag:registerPrefabComponent("models/flags/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_3_TopPart/Flag" .. _colorFbxName .. "Emitter", {
	DataType = "COMP_PARTICLE_EMITTER",
	ParticleSystem = "STONE_GATE_V5_FLAG" .. _colorAssetId .. "_PARTICLE_SYSTEM",
	IsPlaying = true,
	IsEmitting = true -- untill the soldier's AssetBuildingFunction
    })

    PoleFlag:registerPrefabComponent("models/flags/poleFlag" .. _colorFbxName .. ".fbx/Prefab/poleFlag/Pole/step_3_TopPart/Flag" .. _colorFbxName .. "Emitter", {
	DataType = "COMP_PARTICLE_EMITTER_TOGGLE",
	Enabled = false
    })
end

registerColor("Red", "_RED")
registerColor("Blue", "_BLUE")
registerColor("Green", "_GREEN")
registerColor("Purple", "_PURPLE")
registerColor("Orange", "_ORANGE")
registerColor("Turquoise", "_TURQUOISE")
--------------------------------------------------------------------------
